#
# Copyright (c) 2022, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# flake8: noqa
from neptune.common.exceptions import (
    EMPTY_STYLES,
    UNIX_STYLES,
    WINDOWS_STYLES,
)
from neptune.legacy.exceptions import (
    CannotResolveHostname,
    DeleteArtifactUnsupportedInAlphaException,
    DeprecatedApiToken,
    DownloadArtifactsUnsupportedException,
    DownloadArtifactUnsupportedException,
    DownloadSourcesException,
    FileNotFound,
    InvalidChannelValue,
    InvalidNeptuneBackend,
    InvalidNotebookPath,
    NeptuneException,
    NeptuneIncorrectImportException,
    NeptuneIncorrectProjectQualifiedNameException,
    NeptuneLibraryNotInstalledException,
    NeptuneMissingApiTokenException,
    NeptuneMissingProjectQualifiedNameException,
    NeptuneNoExperimentContextException,
    NeptuneUninitializedException,
    NoChannelValue,
    NotADirectory,
    NotAFile,
    UnsupportedClientVersion,
    UnsupportedInAlphaException,
)
